using UnityEngine;
using UnityEngine.InputSystem;// InputValueを使うために必要

public class PlayerController : MonoBehaviour
{
    // 移動速度
    [SerializeField]
    private float moveSpeed = 4.0f;
    //①ここから
    // 回転速度
    [SerializeField]
    private float rotationSpeed = 4.0f;
    //①ここまで
    // 入力のあった移動ベクトル
    private Vector2 moveAmount;
    // 移動のために追加したRigidbody2Dコンポーネントへの参照
    private Rigidbody2D rigidbody2d;

    //③ここから
    // 弾の速度
    [SerializeField]
    private float bulletSpeed = 8.0f;
    // 弾のプレハブ 
    [SerializeField]
    private Bullet bulletPrefab;
    // プレイヤー番号
    private int playerIndex;
    //③ここまで

    //⑥ここから
    //爆発のプレハブ  
    [SerializeField]
    private GameObject explosionPrefab;
    //⑥ここまで

    // 最初に一度呼び出されます
    private void Awake()
    {
        rigidbody2d = GetComponent<Rigidbody2D>();
        //④
        playerIndex = GetComponent<PlayerInput>().playerIndex;// ※2
    }
    // 入力に変化があったときに呼び出されます
    // Player Inputコンポーネントから、"On"+"アクション名"(先頭大文字)の名前の関数が呼ばれる決まりです
    void OnMove(InputValue value)
    {
        moveAmount = value.Get<Vector2>();
    }
    //⑤ここから
    // fireボタンを押したときに呼び出されます
    void OnFire()
    {
        Bullet bullet = Instantiate<Bullet>(bulletPrefab, transform.position, transform.rotation);
        bullet.Move(transform.up * bulletSpeed, playerIndex);
    }
    //⑤ここまで
    // 一定間隔で呼び出されるので、rigidbody2dでの移動を行います
    void FixedUpdate()
    {
        // 現在の座標
        Vector2 currentPosition = rigidbody2d.position;
        // 入力されたベクトルから移動量を求めます
        // Time.fixedDeltaTimeは前回FixedUpdate()が呼ばれてからの経過時間です
        Vector2 deltaPosition = moveAmount * moveSpeed * Time.fixedDeltaTime;
        // 現在の座標に移動量を加算して、新しい座標を計算します
        Vector2 newPosition = currentPosition + deltaPosition;
        // ゲームオブジェクトを新しい座標に移動します
        rigidbody2d.MovePosition(newPosition);
        //②ここから
        //移動量が微小のときは誤差が出るので向きを変えません
        if (moveAmount.sqrMagnitude >= 0.001f)//(a)
        {
            // アークタンジェントは+Xの方向が0度になります。
            // 宇宙船の画像が上向きなので0度のとき上を向くようにするため-90度しています
            float angleZ = Mathf.Atan2(deltaPosition.y, deltaPosition.x) * Mathf.Rad2Deg - 90f;//(b)
                                                                                               // オイラー角でクォータニオンを計算します
            Quaternion targetRotation = Quaternion.Euler(0, 0, angleZ);//(c)
            // 新しい角度を設定します
            transform.rotation = Quaternion.Lerp(transform.rotation, targetRotation, rotationSpeed * Time.deltaTime);//(d)
        }
        //②ここまで
    }
    //⑦ここから
    //衝突した場合に呼び出されます
    private void OnTriggerEnter2D(Collider2D collision)
    {
        Bullet bullet = collision.gameObject.GetComponent<Bullet>();
        // 衝突相手がBulletで、自分の弾でない場合は爆発する
        if (bullet != null && bullet.playerIndex != playerIndex)
        {
            //⑧ここから
            HudManager hudManager = GameObject.FindGameObjectWithTag("HudManager").GetComponent<HudManager>();// ※4
            hudManager.AddScoreText(bullet.playerIndex);
            //⑧ここまで

            // 爆発のプレハブを生成します
            Instantiate<GameObject>(explosionPrefab, transform.position, Quaternion.identity);
            //自分は破棄する
            Destroy(gameObject);
        }
    }
    //⑦ここまで
}